/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  directory statistics
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <descrip.h>
#include <ssdef.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <smg$routines.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define SWING_SAVE_SNAPSHOT_FILE	"SWING$SAVE:SNAPSHOT_XXXXXX.LIS"
#define SYS_LOGIN_SNAPSHOT_FILE		"SYS$LOGIN:SNAPSHOT__XXXXXX.LIS"

static FILE *snap_fp;
static int snap_display;

static void end_say_snapshot(void);
static int  say_snapshot(char *filename);
static void snap_line(struct dsc$descriptor_s *line_desc, int arg);


void snap_line(struct dsc$descriptor_s *line_desc, int arg)
{
    fprintf(snap_fp,
	    "%.*s\n",
	    line_desc->dsc$w_length,
	    line_desc->dsc$a_pointer);
}

int snapshot(void)
{
    char filename[SPEC_SIZE+1];
    char *p;

    strcpy(filename,SWING_SAVE_SNAPSHOT_FILE);
    p = mktemp(filename);
    if (p)
        snap_fp = fopen(p,"w","fop=cbt","mbc=3");
    if (snap_fp == NULL)
    {
        strcpy(filename,SYS_LOGIN_SNAPSHOT_FILE);
        p = mktemp(filename);
        if (p)
            snap_fp = fopen(p,"w","fop=cbt","mbc=3");

        if (snap_fp == NULL)
        {
            sprintf(junk,
		    "Cannot create snapshot file %s",
		    SYS_LOGIN_SNAPSHOT_FILE);
            warning(junk);
	    return(TRUE);
        }
    }

    smg$put_pasteboard(&paste,		/* pasteboard-id */
		       snap_line,	/* action-routine */
		       0,		/* argument */
		       0);		/* flags */

    fgetname(snap_fp,filename);

    say_snapshot(filename);

    fclose(snap_fp);

    end_say_snapshot();
}

/*
**  tell user his screen's been snapshotted
*/
static int say_snapshot(char *filename)
{
    int status,rows,columns;
    int x,y;

    rows    = 1;
    columns = strlen(filename);

    if (!snap_display)
    {
        /*
        **  create display with just one line 
        */
        status = smg$create_virtual_display(&rows,
                                            &columns,
                                            &snap_display,
                                            &SMG$M_BORDER,
                                            0);
        if((status & 1) != 1)
	    return(TRUE);

        status = smg$label_border(&snap_display,
    			          DSC1(" SNAPSHOT "),
                                  &SMG$K_TOP,
			          0,
			          &SMG$M_REVERSE);
    }

    smg$set_cursor_abs(&snap_display,
		        &1,
		        &1);

    smg$put_line(&snap_display,
		 DSC1(filename),
		 &0);
    
    y = page_length / 2;
    x = (SCREEN_WIDTH - columns)/2 + 1;

    smg$paste_virtual_display(&snap_display,
			      &paste,
			      &y,
			      &x,
			      0);

    ding(snap_display);
}

static void end_say_snapshot(void)
{
    sleep(3);

    smg$unpaste_virtual_display(&snap_display,
				&paste);
}
